@@ -1,13 +1,15 @@
 #' Get Wikidata Item ID
 #'
-#' @import jsonlite httr
-#' @importFrom stats rgamma
-#' @importFrom utils URLencode URLdecode
+#' Search www.wikidata.org for wikidata item identifiers.  Note that this search
+#' is currently not limited to chemical substances, so be sure to check your
+#' results.
+#'
 #' @param query character; The searchterm
 #' @param language character; the language to search in
-#' @param match character; How should multiple hits be handeled? 'all' returns all matched IDs,
-#' 'first' only the first match, 'best' the best matching (by name) ID, 'ask' is a interactive mode and the user is asked for input,
-#' 'na' returns NA if multiple hits are found.
+#' @param match character; How should multiple hits be handeled? 'all' returns
+#'   all matched IDs, 'first' only the first match, 'best' the best matching (by
+#'   name) ID, 'ask' is a interactive mode and the user is asked for input, na'
+#'   returns NA if multiple hits are found.
 #' @param verbose logical; print message during processing to console?
 #'
 #' @return if match = 'all' a list with ids, otherwise a dataframe with 4 columns:
@@ -15,7 +17,12 @@
 #'
 #' @note Only matches in labels are returned.
 #'
-#' @author Eduard Szoecs, \email{eduardszoecs@@gmail.com}
+#' @author Eduard Szöcs, \email{eduardszoecs@@gmail.com}
+#'
+#' @import jsonlite httr
+#' @importFrom stats rgamma
+#' @importFrom utils URLencode URLdecode
+#' @importFrom purrr map_df
 #' @export
 #' @examples
 #' \dontrun{
@@ -27,103 +34,76 @@
 #' comps <- c('Triclosan', 'Glyphosate')
 #' get_wdid(comps)
 #' }
-get_wdid <- function(query, language = 'en', match = c('best', 'first', 'all', 'ask', 'na'),
-                     verbose = TRUE){
+get_wdid <-
+  function(query,
+           match = c('best', 'first', 'all', 'ask', 'na'),
+           verbose = TRUE,
+           language = 'en') {
+
   # language <-  'en'
   # query <- 'Triclosan'
+
   match <- match.arg(match)
   foo <- function(query, language, match, verbose){
-    query <- URLencode(query)
-    limit <-  50
-    qurl <- paste0("wikidata.org/w/api.php?action=wbsearchentities&format=json&type=item")
-    qurl <- paste0(qurl, "&language=", language, "&limit=", limit, "&search=", query)
-    if (verbose)
-      message('Querying ', qurl)
-    Sys.sleep(0.3)
-    cont <- fromJSON(content(GET(qurl, user_agent('webchem (https://github.com/ropensci/webchem)')), 'text'))
-    search <- cont$search
-    if (length(search) == 0) {
-      if (verbose)
-        message('Substance not found! Returing NA. \n')
+    if (is.na(query)){
       id <- NA
-      attr(id, "matched") <- NA
-      attr(id, "distance") <- NA
-      return(id)
-    }
-    # use only matches on label
-    search <- search[search$match$type %in% c('label', 'alias'), ]
-    # # check matches
-    search <- search[tolower(iconv(search$match$text,
-                                   "latin1",
-                                   "ASCII",
-                                   sub = "")) == tolower(URLdecode(query)), ]
-
-    if (nrow(search) > 1) {
+      matched_sub <- NA
+    } else {
+      query1 <- URLencode(query)
+      limit <- 50
+      qurl <-
+        paste0("wikidata.org/w/api.php?action=wbsearchentities&format=json&type=item")
+      qurl <- paste0(qurl, "&language=", language, "&limit=", limit, "&search=", query1)
       if (verbose)
-        message("More then one Link found. \n")
-      if (match == 'na') {
+        message('Querying ', qurl)
+      Sys.sleep(0.3)
+      cont <-
+        fromJSON(content(GET(
+          qurl,
+          user_agent('webchem (https://github.com/ropensci/webchem)')
+        ), 'text'))
+      search <- cont$search
+      if (length(search) == 0) {
         if (verbose)
-          message("Returning NA. \n")
+          message('Substance not found! Returing NA. \n')
         id <- NA
         matched_sub <- NA
-        d <- NA
-      }
-      if (match == 'all') {
-        if (verbose)
-          message("Returning all matches. \n")
-        id <- search$id
-        matched_sub <- search$label
-        d <- 'all'
-      }
-      if (match == 'first') {
-        if (verbose)
-          message("Returning first match. \n")
-        id <- search$id[1]
-        matched_sub <- search$label[1]
-        d <- 'first'
-      }
-      if (match == 'best') {
-        if (verbose)
-          message("Returning best match. \n")
-        dd <- adist(URLdecode(query), search$label) / nchar(search$label)
-        id <- search$id[which.min(dd)]
-        d <- round(dd[which.min(dd)], 2)
-        matched_sub <- search$label[which.min(dd)]
-      }
-      if (match == 'ask') {
-        tochoose <- data.frame(match = search$label, url = search$url, description = search$description)
-        print(tochoose)
-        message("\nEnter rownumber of compounds (other inputs will return 'NA'):\n") # prompt
-        take <- as.numeric(scan(n = 1, quiet = TRUE))
-        if (length(take) == 0) {
-          id <- NA
-          matched_sub <- NA
-          d <- NA
-        }
-        if (take %in% seq_len(nrow(tochoose))) {
-          id <- search$id[take]
-          matched_sub <- search$label[take]
-          d <- 'interactive'
+      } else {
+        # use only matches on label
+        search <- search[search$match$type %in% c('label', 'alias'), ]
+        # # check matches
+        search <- search[tolower(iconv(search$match$text,
+                                       "latin1",
+                                       "ASCII",
+                                       sub = "")) == tolower(query), ]
+
+        if (nrow(search) > 1) {
+          id <-
+            matcher(
+              search$id,
+              query = query,
+              result = search$label,
+              match = match,
+              verbose = verbose
+            )
+          matched_sub <- names(id)
+        } else {
+          id <- search$id
+          matched_sub <- search$label
         }
       }
-    } else {
-      id <- search$id
-      d <- 0
-      matched_sub <- search$label
     }
-    names(id) <- NULL
-    attr(id, "matched") <- matched_sub
-    attr(id, "distance") <- d
-    return(id)
-  }
-  out <- lapply(query, foo, language = language, match = match, verbose = verbose)
-  if (match != 'all') {
-    out <- data.frame(t(sapply(out, function(y) {
-      c(y, attr(y, 'matched'), attr(y, 'distance'))
-    })), stringsAsFactors = FALSE)
-    names(out) <- c('id', 'match', 'distance')
-    out[['query']] <- query
+    out <- tibble(query = query, match = matched_sub, wdid = id)
+    return(out)
   }
+  out <-
+    purrr::map_df(query,
+           ~ foo(
+             query = .x,
+             match = match,
+             language = language,
+             verbose = verbose
+           ))
   return(out)
 }
 
@@ -134,6 +114,7 @@ get_wdid <- function(query, language = 'en', match = c('best', 'first', 'all', '
 #' Retrieve Indentifiers from Wikidata
 #'
 #' @import jsonlite
+#' @import httr
 #' @importFrom stats rgamma
 #'
 #' @param id character; identifier, as returned by \code{\link{get_wdid}}
@@ -153,7 +134,7 @@ get_wdid <- function(query, language = 'en', match = c('best', 'first', 'all', '
 #' Mitraka, Elvira, Andra Waagmeester, Sebastian Burgstaller-Muehlbacher, et al. 2015
 #' Wikidata: A Platform for Data Integration and Dissemination for the Life Sciences and beyond. bioRxiv: 031971.
 #'
-#' @author Eduard Szoecs, \email{eduardszoecs@@gmail.com}
+#' @author Eduard Szöcs, \email{eduardszoecs@@gmail.com}
 #' @export
 #' @examples
 #' \dontrun{
@@ -189,7 +170,8 @@ wd_ident <- function(id, verbose = TRUE){
     Sys.sleep( rgamma(1, shape = 15, scale = 1/10))
     if (verbose)
       message('Querying ', qurl)
-    tmp <- fromJSON(qurl)
+    res <- GET(qurl)
+    tmp <- fromJSON(content(res, as = "text"))
 
     vars_out <- tmp$head$vars
     out <- tmp$results$bindings
@@ -227,4 +209,3 @@ wd_ident <- function(id, verbose = TRUE){
   class(out) <- c('wd_ident', 'data.frame')
   return(out)
 }
-
